/* entry_ref_history.c--entry_ref_history implementation
 * Rob Earhart
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Carnegie Mellon
University not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.

CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE FOR
ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
******************************************************************/

#include <config.h>
#include <stdlib.h>
#include <errno.h>
#include "entry_ref_history.h"

result_t
entry_ref_history_alloc(stamp_t stamp,
			const str_t * db_path,
			dataset_key_t dataset_key,
			entry_ref_history_t ** dest)
{
  result_t        result;

  I(dest);

  *dest = (entry_ref_history_t *) malloc(sizeof(entry_ref_history_t));
  if (!*dest)
    return errno;

  *(entry_ref_history_STAMP(*dest)) = stamp;
  entry_ref_history_DBPATH(*dest) = NULL;
  if (db_path) {
    result = str_copy(db_path, &entry_ref_history_DBPATH(*dest));
    if (result != OK)
      goto cleanup;
  }
  entry_ref_history_DBKEY(*dest) = dataset_key;

  (*dest)->next = NULL;

  return OK;

cleanup:
  free(*dest);
  *dest = NULL;
  return result;
}

result_t
entry_ref_history_free(entry_ref_history_t ** history)
{
  I(history);
  I(*history);

  if (entry_ref_history_DBPATH(*history))
    str_free(&entry_ref_history_DBPATH(*history));
  free(*history);
  *history = NULL;

  return OK;
}
