/* respcode.h--response code interface
 * Rob Earhart
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Carnegie Mellon
University not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.

CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE FOR
ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
******************************************************************/

#ifndef RESPCODE_H
#define RESPCODE_H

/* A response code is used to convey machine-parseable information to
 * a client in a status response.  This provides a general interface
 * for respcodes, and some functions for creating them. */

#include "strlist.h"
#include "../util/str.h"

typedef struct respcode_s respcode_t;

struct client_s;

/* Write out a response code... */
extern          result_t
respcode_write(respcode_t * code,
	       struct client_s * client,
	       result_t *command_result);

extern result_t respcode_free(respcode_t ** code);

/* This allocates a NOEXIST response code.  It takes ownership
 * of the path's memory. */
extern result_t
respcode_noexist_alloc(str_t * path,
		       respcode_t ** dest);

/* This allocates a WAYTOOMANY response code. */
extern result_t respcode_waytoomany_alloc(respcode_t ** dest);

/* This allocates a TOOMANY response code. */
extern result_t
respcode_toomany_alloc(unsigned n_entries,
		       respcode_t ** dest);

extern result_t
respcode_refer_alloc(const strlist_t * strlist,
		     respcode_t ** dest);

#endif				/* RESPCODE_H */
