/* stamp.h - timestamp interface
 * Rob Earhart
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Carnegie Mellon
University not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.

CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE FOR
ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
******************************************************************/

#ifndef STAMP_H
#define STAMP_H

#include <time.h>

typedef struct {
  time_t          sec;		/* network byte order */
  uint32_t        frac;		/* network byte order */
}               stamp_t;

#define STAMP_ZERO_INITIALIZER {0,0}

#define stamp_SET(stamp, s, f) ((stamp)->sec = htonl(s),	\
				(stamp)->frac = htonl(f))
#define stamp_COPY(dest, src) ((dest)->sec = (src)->sec,	\
			       (dest)->frac = (src)->frac)
#define stamp_SEC(stamp) ((time_t)(ntohl((stamp)->sec)))
#define stamp_FRAC(stamp) (ntohl((stamp)->frac))
#define stamp_INCR_FRAC(stamp) ((stamp)->frac = htonl(ntohl((stamp)->frac) + 1))
#define stamp_LT(a, b) (stamp_SEC(a) < stamp_SEC(b)		\
			|| (stamp_SEC(a) == stamp_SEC(b)	\
			    && stamp_FRAC(a) < stamp_FRAC(b)))
#define stamp_EQ(a, b) (stamp_SEC(a) == stamp_SEC(b)		\
			&& stamp_FRAC(a) == stamp_FRAC(b))
#endif				/* STAMP_H */
