/* strlist.h - interface to string lists
 * Rob Earhart
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Carnegie Mellon
University not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.

CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE FOR
ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
******************************************************************/

#ifndef STRLIST_H
#define STRLIST_H

#include "../util/str.h"

/* Strlists are a simple array of str objects.  What makes them
 * interesting is that their pointers are self-referential, so they
 * may be moved around, mmap()ed, etc.  They're created via the
 * entry code in entry.c. */

typedef struct {
  uint32_t        len;		/* always in network byte order */
  /* immediately followed by uint32_t offsets, in NBO */
}               strlist_t;

typedef struct {
  const strlist_t *strlist;
  uint32_t       *str;
}               strlist_iterator_t;

#define strlist_N_VALUES(s) (ntohl((s)->len))
#define strlist_OFFSETS(s) ((uint32_t *)((s) + 1))
#define strlist_NTH_OFFSET(s, n) (strlist_OFFSETS(s) /* base ptr... */	    \
				  + strlist_N_VALUES(s) /* go to the end */ \
				  - 1 /* NValues==1 => value 0 */	    \
				  - (n) /* back up by N... */		    \
				  )
#define strlist_NTH_VALUE(s, n) ((str_t *)((caddr_t)strlist_NTH_OFFSET(s,n) \
					   + ntohl(*strlist_NTH_OFFSET(s,n))))
#define strlist_ITER_BEGIN(s, i) ((i)->strlist = (s),			\
				  ((i)->str = strlist_OFFSETS(s)	\
				   + strlist_N_VALUES(s) - 1))
#define strlist_ITER_ATEND(i) ((i)->str <= strlist_OFFSETS((i)->strlist) - 1)
#define strlist_ITER_INCR(i) ((i)->str--)
#define strlist_ITER_VALUE(i) ((str_t *)((caddr_t)((i)->str)	\
 			    		 + ntohl(*((i)->str))))
#endif				/* STRLIST_H */
