/* traverse.h - interface to ACAP path traversal code
 * Rob Earhart
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Carnegie Mellon
University not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.

CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE FOR
ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
******************************************************************/

#ifndef TRAVERSE_H
#define TRAVERSE_H

/* This resolves paths to their corresponding dataset_ref_t
 * objects. */

#include <pthread.h>
#include "../util/str.h"
#include "stamp.h"
#include "dataset_ref.h"
#include "trx.h"
#include "client.h"
#include "respcode.h"

struct entry_ref_s;

struct dataset_ref_s;

/* Note: When this returns OK, the dataset_ref will be referenced.
 * It is imperative that the caller unreference it. */
extern          result_t
traverse_read(const str_t * path,
	      client_t * client,/* for ~ expansion */
	      trx_t * trx,	/* NULL => no create intermediaries */
	      unsigned *inherit_level,	/* Keeps track of inheritance level */
	      struct dataset_ref_s ** dest_dataset,
	      str_t ** inherit /* For inheritance calculation */ ,
	      respcode_t ** respcode);

/* Note: When this returns OK, the entry_ref will be referenced and
 * exclusively locked.  It is imperative that the caller unlock
 * and unreference it. */

/* Note, also, that this will be replaced by a simultaneous traverse
 * at some later point in time. XXXXXXX */

typedef enum {
  TRAVERSE_NO_CREATE_DATASET,
  TRAVERSE_CREATE_DATASET
}               traverse_create_t;

extern result_t
traverse_open(dataset_key_t key,
	      trx_t * trx,
	      struct dataset_ref_s ** dest);

extern result_t
traverse_get_dataset(struct entry_ref_s * entry_ref,
		     trx_t * trx,
		     traverse_create_t create,
		     struct dataset_ref_s ** dest,
		     respcode_t ** respcode);

extern result_t
traverse_write(const str_t * path,
	       client_t * client,	/* for ~ expansion */
	       trx_t * trx,	/* NULL => no create intermediaries */
	       traverse_create_t create,
	       struct entry_ref_s ** dest,
	       respcode_t ** respcode);

#endif				/* TRAVERSE_H */
